require 'globals.dungeon_creator.dynamic_windows.dungeon_editor_window'
require 'globals.dungeon_creator.generation.dungeon_generation'

function CreateDungeonArray(dungeonArray)
	for x=1,24 do
		for y=1,20 do
			local index = 
			{
				CellX=x,
				CellY=y,
				ClientId=0,
				TileImageList={North="HueSwatch",West="HueSwatch",South="HueSwatch",East="HueSwatch"},
				TileImageData={Description="",Tooltip=""},
				TileSet="BoneAndDirt",
				Rotation="North",
				RealTile=0,
				Location = Loc(0,0,0),
				Spawners = {},
				Chests = {},
				LootTables = {},
				Doors = {},
				Decorations = {},
				SpawnPoints = {},
			}
			table.insert(dungeonArray,index)			
		end
	end	
	return dungeonArray
end

function SetMode(editorInfo,mode)
	local newInfo = editorInfo
	for k,v in pairs(newInfo.Mode) do
		newInfo.Mode[k] = false
	end
	newInfo.Mode[mode] = true
	this:SetObjVar("Data.EditorInfo",newInfo)
end



----------------------------------------------------------
--EVENTTYPE - ModuleAttached 
----------------------------------------------------------
RegisterEventHandler(EventType.ModuleAttached, GetCurrentModule(), function() 
	--AddUseCase(this,"[FF9900]Controller Menu[-]",false)
	AddUseCase(this,"[FF9900]Edit Dungeon Menu[-]",true)
	AddUseCase(this,"[FF9900]Refresh Modules[-]",false)

	local editorInfo = 
	{
		SelectedPaintTileInfo=TileInfo["BoneAndDirt"].Tiles._695,
		CurrentSelectedTile = {CellX=0,CellY=0,ClientId=0},
		TileSet="BoneAndDirt",
		DungeonArray = {},
		Mode = {Draw=true,Encounters=false,Configuration=false}, -- Draw, Encounters, Configuration
	}

	editorInfo.DungeonArray = CreateDungeonArray({})
	this:SetObjVar("Data.EditorInfo",editorInfo)
end)

----------------------------------------------------------
--EVENTTYPE - DynamicWindowResponse 
----------------------------------------------------------
RegisterEventHandler(EventType.DynamicWindowResponse,"DungeonEditor",
function (user,buttonId,fieldData)
	local data = StringSplit(buttonId,"|")
	local editorInfo = this:GetObjVar("Data.EditorInfo") or nil
	if(data[1]=="ToggleMode") then
		SetMode(editorInfo,data[2])
		ShowDungeonEditorWindow(user,this)
	end
	if(editorInfo.Mode["Draw"] == true) then
		if(data[1]=="OnCellClick") then
			if not(editorInfo == nil) then
				editorInfo.CurrentSelectedTile = {CellX=tonumber(data[2]),CellY=tonumber(data[3]),ClientId=editorInfo.SelectedPaintTileInfo.ClientId}
				local dungeonArray = editorInfo.DungeonArray
				for k,v in pairs(dungeonArray) do
					if(v.CellX == tonumber(data[2]) and v.CellY == tonumber(data[3])) then
						v.ClientId=editorInfo.SelectedPaintTileInfo.ClientId
						v.TileImageList = editorInfo.SelectedPaintTileInfo.IconImage
						v.TileImageData.Description = editorInfo.SelectedPaintTileInfo.Description
						v.TileImageData.Tooltip = editorInfo.SelectedPaintTileInfo.Tooltip
						v.TileSet = "BoneAndDirt"
						v.Template = editorInfo.SelectedPaintTileInfo.Template
					end
				end
				editorInfo.DungeonArray = dungeonArray
				this:SetObjVar("Data.EditorInfo",editorInfo)
				ShowDungeonEditorWindow(user,this)
			end
		end
		if(data[1]=="SelectNewTile") then
			local tilekey = data[2]
			if not(editorInfo == nil) then
				editorInfo.SelectedPaintTileInfo = TileInfo["BoneAndDirt"].Tiles[tilekey]
				this:SetObjVar("Data.EditorInfo",editorInfo)
				ShowDungeonEditorWindow(user,this)
			end
		end
	end
	if(editorInfo.Mode["Configuration"] == true) then
		if(data[1]=="OnCellClick" and editorInfo.Mode["Configuration"]==true) then
			if not(editorInfo == nil) then
				local clickedTile = GetTileInfoAtSelectedCell(editorInfo)
				if(clickedTile ~= false) then
					editorInfo.CurrentSelectedTile = {CellX=tonumber(data[2]),CellY=tonumber(data[3]),ClientId=clickedTile.ClientId}
					this:SetObjVar("Data.EditorInfo",editorInfo)
					ShowDungeonEditorWindow(user,this)
				end
			end
		end
		if(data[1]=="RotateTileLeft90") then
			RotateCell(this,"Left")
			ShowDungeonEditorWindow(user,this)
		end
		if(data[1]=="RotateTileRight90") then
			RotateCell(this,"Right")
			ShowDungeonEditorWindow(user,this)
		end		
		if(data[1]=="GenerateDungeon") then
			GenerateDungeon(this)
			ShowDungeonEditorWindow(user,this)
		end		
	end
end)

----------------------------------------------------------
--EVENTTYPE - LoadedFromBackup 
----------------------------------------------------------
RegisterEventHandler(EventType.LoadedFromBackup,"",function()

end)

----------------------------------------------------------
--EVENTTYPE - Message "UseObject" 
----------------------------------------------------------
RegisterEventHandler(EventType.Message, "UseObject", function(user,usedType)
	if(usedType=="[FF9900]Edit Dungeon Menu[-]") then
		ShowDungeonEditorWindow(user,this)
    end
    --ReloadModule("region")
    if(usedType=="[FF9900]Refresh Modules[-]") then
		ReloadModule("globals.dungeon_creator.dynamic_windows.dungeon_editor_window")
		ReloadModule("globals.dungeon_creator.controllers.dungeon_controller")
    end
end)

