---------------------------------------------------------------------------------
--						DUNGEON EDITOR MENU
---------------------------------------------------------------------------------

---------------------------------------------------------------------------------
-- Dungeon Editor Configuration
---------------------------------------------------------------------------------
DungeonEditorStyle =
{
	ScrollElementBackgroundColor = "A97F54",
	ScrollElementBackgroundColorSelected = "763B21",
	GridBackgroundColor = "A97F54",
	GridBorderColor = "763B21",
	PropertyBackgroundColor = "763B21",
	DrawMenuIconActiveColor = "008000",
	DrawMenuIconDeactiveColor = "800000",
	SelectedTileColor = "FF5555"
}

DungeonEditorLocalization =
{
	ConfigurationModeActive = "[FF7700]Configuration Mode\n[FFFF00]configuration mode allows you to select tiles on the map, and use the property window to adjust the tiles characteristics.",
	ConfigurationModeInactive = "[FF7700]Enable Configuration Mode\n[FFFF00]click this button to toggle on Configuration Mode. this will allow you to change the dungeon tile characteristics.",
	DrawModeActive = "[FF7700]Draw Mode\n[FFFF00]draw mode allows you to place tiles on the map, and use the property window to rotate and adjust them.",
	DrawModeInactive = "[FF7700]Enable Draw Mode\n[FFFF00]click this button to toggle on Draw Mode. this will allow you to change the dungeon tiles.",
	EncountersModeActive = "[FF7700]Encounters Mode\n[FFFF00]encounters mode allows you to create things players can encounter, and use the property window to set up the encounter type.",
	EncountersModeInactive = "[FF7700]Enable Encounters Mode\n[FFFF00]click this button to toggle on Encounters Mode. this will allow you to use the property window to adjust the encounter characteristics.",
	SelectedTile = "[FFFFFF]Selected Tile",
	DrawTileButtonTooltip = "[FF7700]Click Tile\n[FFFF00]this will add the current selected paint tile to this slot at index",
	ConfigurationTileButtonTooltip = "[FF7700]Click Tile\n[FFFF00]this will allow you to edit this tiles characteristics in the property window",
	EncountersTileButtonTooltip = "[FF7700]Click Tile\n[FFFF00]this will allow you to edit this tiles various encounter types in the property window",
	RotateTileLeft90Tooltip = "[FF7700]Rotate 90 CCW\n[FFFF00]this will rotate the selected tile by 90 degrees counter clockwise",
	RotateTileRight90Tooltip = "[FF7700]Rotate 90 CW\n[FFFF00]this will rotate the selected tile by 90 degrees clockwise",
	GenerateDungeonTooltip = "[FF7700]Generate Dungeon\n[FFFF00]this will destroy all current dungeon tiles, and generate the dungeon again",
}

---------------------------------------------------------------------------------
-- Dungeon Editor Dynamic Window Functions
---------------------------------------------------------------------------------
function DrawTile(cellX,cellY,dynamicWindow,tileId,controllerInfo)
	if(tileId == "HueSwatch") then
		dynamicWindow:AddImage(cellX*36,cellY*36+30,tileId,36,36,nil,DungeonEditorStyle.GridBorderColor)
		dynamicWindow:AddImage(cellX*36+2,cellY*36+2+30,tileId,32,32,"Sliced",DungeonEditorStyle.GridBackgroundColor)
	else
		local color = "FFFFFF"
		if(cellX == controllerInfo.CurrentSelectedTile.CellX and cellY == controllerInfo.CurrentSelectedTile.CellY) then color = DungeonEditorStyle.SelectedTileColor end
		dynamicWindow:AddImage(cellX*36,cellY*36+30,tileId,36,36,"Simple",color)
	end
end

function DrawCloseButton(x,y,dynamicWindow)
	dynamicWindow:AddImage(x,y,"HueSwatch",24,24,nil,"AA0000")
	dynamicWindow:AddImage(x+2,y+2,"HueSwatch",20,20,nil,"770000")
	dynamicWindow:AddButton(x,y,"CloseDungeonEditor","",24,24,"[FF7700]Exit Dungeon Editor\n[FFFF00]closes the dungeon editor","",true,"Invisible")
end

function DrawDungeon(dynamicWindow,controllerInfo)
	for k,v in pairs(controllerInfo.DungeonArray) do
		DrawTile(v.CellX,v.CellY,dynamicWindow,v.TileImageList[v.Rotation],controllerInfo)
		if(controllerInfo.Mode["Draw"] == true) then
			dynamicWindow:AddButton(v.CellX*36,v.CellY*36+30,"OnCellClick|"..v.CellX.."|"..v.CellY.."|","",36,36,DungeonEditorLocalization.DrawTileButtonTooltip.."\n\n("..v.CellX..","..v.CellY..")","",false,"Invisible")
		end
		if(controllerInfo.Mode["Configuration"] == true) then
			dynamicWindow:AddButton(v.CellX*36,v.CellY*36+30,"OnCellClick|"..v.CellX.."|"..v.CellY.."|","",36,36,DungeonEditorLocalization.ConfigurationTileButtonTooltip.."\n\n("..v.CellX..","..v.CellY..")","",false,"Invisible")
		end
		if(controllerInfo.Mode["Encounters"] == true) then
			dynamicWindow:AddButton(v.CellX*36,v.CellY*36+30,"OnCellClick|"..v.CellX.."|"..v.CellY.."|","",36,36,DungeonEditorLocalization.EncountersTileButtonTooltip.."\n\n("..v.CellX..","..v.CellY..")","",false,"Invisible")
		end
	end
end

function RotateCell(dungeonController,direction)
	local editorInfo = dungeonController:GetObjVar("Data.EditorInfo") or nil
	local array = editorInfo.DungeonArray
	local newRotation = "North"
	for k,v in pairs(array) do
		if(editorInfo.CurrentSelectedTile.CellX==v.CellX and editorInfo.CurrentSelectedTile.CellY==v.CellY) then
			if(direction == "Left") then
				if(v.Rotation == "South") then newRotation = "East" end
				if(v.Rotation == "West") then newRotation = "South" end
				if(v.Rotation == "North") then newRotation = "West" end
				if(v.Rotation == "East") then newRotation = "North" end
				v.Rotation = newRotation
			end
			if(direction == "Right") then
				if(v.Rotation == "South") then newRotation = "West" end
				if(v.Rotation == "West") then newRotation = "North" end
				if(v.Rotation == "North") then newRotation = "East" end
				if(v.Rotation == "East") then newRotation = "South" end
				v.Rotation = newRotation
			end			
		end
	end
	editorInfo.DungeonArray = array
	dungeonController:SetObjVar("Data.EditorInfo",editorInfo)
end

function GetTileInfoAtSelectedCell(controllerInfo)
	for k,v in pairs(controllerInfo.DungeonArray) do
		if(controllerInfo.CurrentSelectedTile.CellX==v.CellX and controllerInfo.CurrentSelectedTile.CellY==v.CellY) then
			return v
		end
	end
	return false
end

function DrawPropertiesWindow(x,y,dynamicWindow,controllerInfo)
	local currentTile = GetTileInfoAtSelectedCell(controllerInfo)
	dynamicWindow:AddImage(x,y,"dungeon_editor_background",250,820)
	dynamicWindow:AddImage(x+10,y+25,"HueSwatch",230,770,nil,DungeonEditorStyle.PropertyBackgroundColor)	
	if(currentTile ~=nil and currentTile ~= false) then
		dynamicWindow:AddLabel(x+125,y+30,DungeonEditorLocalization.SelectedTile .." ("..currentTile.CellX..","..currentTile.CellY..")",250,28,28,"center")
		dynamicWindow:AddImage(x+65,y+65,currentTile.TileImageList[currentTile.Rotation],120,120,"Simple")
		dynamicWindow:AddLabel(x+120,y+115,"[FF0000]"..tostring(controllerInfo.CurrentSelectedTile.ClientId),120,120,28,"center")
		dynamicWindow:AddLabel(x+25,y+220,currentTile.TileImageData.Description,210,60,18,"left")
		if(controllerInfo.Mode["Configuration"] == true) then
			dynamicWindow:AddButton(x+16,y+65,"RotateTileLeft90","-90",40,30,DungeonEditorLocalization.RotateTileLeft90Tooltip,"",false)
			dynamicWindow:AddButton(x+191,y+65,"RotateTileRight90","+90",40,30,DungeonEditorLocalization.RotateTileRight90Tooltip,"",false)

			dynamicWindow:AddButton(x+16,y+760,"GenerateDungeon","Generate Dungeon",221,30,DungeonEditorLocalization.GenerateDungeonTooltip,"",false)
		end
	end	
end

function DrawMenu(dynamicWindow,controllerInfo)
	local DrawMode = controllerInfo.Mode["Draw"] 
	local ConfigurationMode = controllerInfo.Mode["Configuration"] 
	local EncountersMode = controllerInfo.Mode["Encounters"]
	if(DrawMode) then
		dynamicWindow:AddImage(36,24,"icon_build_dungeon_normal",40,40,"Simple",DungeonEditorStyle.DrawMenuIconActiveColor)
		dynamicWindow:AddButton(36,24,"NoResponseNeeded","",40,40,DungeonEditorLocalization.DrawModeActive,"",false,"Invisible")
	else
		dynamicWindow:AddImage(36,24,"icon_build_dungeon_disabled",40,40,"Simple",DungeonEditorStyle.DrawMenuIconDeactiveColor)
		dynamicWindow:AddButton(36,24,"ToggleMode|Draw","",40,40,DungeonEditorLocalization.DrawModeInactive,"",false,"Invisible")
	end
	if(ConfigurationMode) then
		dynamicWindow:AddImage(78,24,"icon_portals_normal",40,40,"Simple",DungeonEditorStyle.DrawMenuIconActiveColor)
		dynamicWindow:AddButton(78,24,"NoResponseNeeded","",40,40,DungeonEditorLocalization.ConfigurationModeActive,"",false,"Invisible")
	else
		dynamicWindow:AddImage(78,24,"icon_portals_disabled",40,40,"Simple",DungeonEditorStyle.DrawMenuIconDeactiveColor)
		dynamicWindow:AddButton(78,24,"ToggleMode|Configuration","",40,40,DungeonEditorLocalization.ConfigurationModeInactive,"",false,"Invisible")
	end
	if(EncountersMode) then
		dynamicWindow:AddImage(120,24,"icon_spawns_normal",40,40,"Simple",DungeonEditorStyle.DrawMenuIconActiveColor)
		dynamicWindow:AddButton(120,24,"NoResponseNeeded","",40,40,DungeonEditorLocalization.EncountersModeActive,"",false,"Invisible")
	else
		dynamicWindow:AddImage(120,24,"icon_spawns_disabled",40,40,"Simple",DungeonEditorStyle.DrawMenuIconDeactiveColor)
		dynamicWindow:AddButton(120,24,"ToggleMode|Encounters","",40,40,DungeonEditorLocalization.EncountersModeInactive,"",false,"Invisible")
	end
end

function ShowDungeonEditorWindow(user,controller)
	local controllerInfo = controller:GetObjVar("Data.EditorInfo") or nil
	ControllerReference = controller
	local window = DynamicWindow("DungeonEditor","",1000,820,-500,-410,"TransparentDraggable","Center")
	--background
	window:AddImage(1,1,"dungeon_editor_background",1000,820,"Simple")
	DrawCloseButton(954,24,window)
	DrawMenu(window,controllerInfo)

	--if editor is starting up for first time, lets set the
	--editorInfo information
	if not(controllerInfo == nil) then
		DrawDungeon(window,controllerInfo)
	end
	--draw the icon list if your in draw mode
	if(controllerInfo.Mode["Draw"] == true) then
		local scrollWindow = ScrollWindow(910,146,60,640,40)

		for key,tileInfo in pairs(TileInfo["BoneAndDirt"].Tiles) do
			local scrollElement = ScrollElement()
			if(tileInfo.IconImage.North == controllerInfo.SelectedPaintTileInfo.IconImage.North) then
				scrollElement:AddImage(1,1,"HueSwatch",50,40,"Sliced",DungeonEditorStyle.ScrollElementBackgroundColorSelected)
				scrollElement:AddImage(3,3,tileInfo.IconImage.North,36,36,"Simple")
				scrollElement:AddButton(1,1,"SelectNewTile|"..key,"",50,40,"[FF7700]Set Paint Tile\n[FFFF00]click this to set as the painted tile","",false,"Invisible")
			else
				scrollElement:AddImage(1,1,"HueSwatch",50,40,"Sliced",DungeonEditorStyle.ScrollElementBackgroundColor)
				scrollElement:AddImage(3,3,tileInfo.IconImage.North,36,36,"Simple")
				scrollElement:AddButton(1,1,"SelectNewTile|"..key,"",50,40,"[FF7700]Set Paint Tile\n[FFFF00]click this to set as the painted tile","",false,"Invisible")
			end
			scrollWindow:Add(scrollElement)
		end
		window:AddScrollWindow(scrollWindow)
	
		--current selected tile icon
		window:AddImage(912,67,controllerInfo.SelectedPaintTileInfo.IconImage.North,60,60,"Simple")
		window:AddButton(912,67,"NoResponseNeeded","",60,60,"[FF7700]Currently Selected\n[FFFF00]this tile is the currently selected tile which you will paint when you click one of the empty map tiles.","",false,"Invisible")
	end

	DrawPropertiesWindow(1001,0,window,controllerInfo)

	if(controllerInfo.Mode["Encounters"] == true) then
	end
	if(controllerInfo.Mode["Configuration"] == true) then
	end

	user:OpenDynamicWindow(window,controller)
end

