-- written by Gizmo
----------------------------------------------------------------------
-- If you like Project Phoenix and want to help support the project
-- you could send any donation amount to jessebutler1975@gmail.com at
-- paypal.com
--
-- Thank you for using my product, 
-- I hope you enjoy it as much as I do
----------------------------------------------------------------------
Systems_Factions = 
{
	-----------------------------------------------------------------------------------
	-- 						Systems_Factions.AdjustFaction
	-----------------------------------------------------------------------------------
	-- player 	- player in which to adjust the faction for
	-- faction 	- will pass as a string value
	-- value 	- the integer/double to add/subtract
	SetFaction = function (player,faction,value)
		local factionTable = player:GetObjVar("Systems_Factions") or nil
		if(factionTable ~= nil) then
			for k,v in pairs(factionTable) do
				if(v[1]==faction) then
					table.remove(factionTable,k)
					table.insert(factionTable,{faction,value})
					player:SetObjVar("Systems_Factions",factionTable)
				end		
			end
		end
	end,
	-- player 	- player in which to adjust the faction for
	-- faction 	- will pass as a string value
	-- value 	- the integer/double to add/subtract
	-- cap 		- when adding/subtracting never go beyond
	--       	  this value. zero here means no cap.
	AdjustFaction = function (player, faction, value, cap)
		--grab a local copy of our players System.Factions information
		local factionTable = player:GetObjVar("Systems_Factions") or nil
		--if table is nil then create it
		if(factionTable == nil) then 
			player:SetObjVar("Systems_Factions",{}) 
			factionTable = {}
		end
		--if we have any bad values just return
		if(faction == nil or value == nil or cap == nil) then return end		
		---------------------------------------------------------------------------
		--if our integer values are passed as strings then convert
		--them back to integers.
		---------------------------------------------------------------------------
		if(type(value) == "string") then
			value = tonumber(value)
		end
		if(type(cap) == "string") then
			cap = tonumber(cap)
		end
		---------------------------------------------------------------------------
		--if the factionTable is empty then this must be the first entry
		--so just create it and be done with it
		if(#factionTable <= 0) then			
			table.insert(factionTable,{faction,value})
			player:SetObjVar("Systems_Factions",factionTable)
			return
		--otherwise lets update any entry we find accordingly.
		else
			-----------------------------------------------------------------------
			--this whole section just gathers the information about the faction,
			--then it either adds to the faction value or subtract. Then it checks
			--if these values pushed past the cap, if so correct it. Then if it 
			--does not find a record, make one, otherwise remove the old record and
			--replace it with the new updated one.
			-----------------------------------------------------------------------
			local foundRecord = false
			local total = 0
			local recordNumber = -1	
			local factionName = ""
			local factionValue = 0					
			for k,v in pairs(factionTable) do
				factionName = v[1]
				factionValue = v[2]
				if(factionName == faction) then				
					--since we found a record of the faction
					--we will need to update it
					if(value > 0) then
						if(factionValue + value > cap) then
							if(factionValue < cap) then total = cap end
						else
							total = factionValue + value
							if(total > cap) then total = cap end
						end
					elseif(value < 0) then
						total = factionValue + value
					end
					foundRecord = true
					recordNumber = k
				end
			end
			-----------------------------------------------------------------------
			--if no record, then create a new record for this
			--faction
			if not (foundRecord) then
				table.insert(factionTable,{faction,value})
				player:SetObjVar("Systems_Factions",factionTable)
			--otherwise remove the old factionTable record and insert a
			--new record with the updated information
			else
				table.remove(factionTable,recordNumber)
				table.insert(factionTable,{faction,total})
				player:SetObjVar("Systems_Factions",factionTable)				
			end
		end
	end,
	-----------------------------------------------------------------------------------
	-- 						Systems_Factions.RemoveFaction
	-----------------------------------------------------------------------------------
	-- player  		- player to check (object)
	-- faction 		- faction to check and remove(string value)
	RemoveFaction = function(player,faction)
		--grab a local copy of our players System.Factions information
		local factionTable = player:GetObjVar("Systems_Factions") or nil
		--if table is nil then create it
		if(factionTable == nil) then player:SetObjVar("Systems_Factions",{}) end
		--if we have any bad values just return
		if(faction == nil) then return end
		--if the faction table is empty, then this must be the first entry 
		--so treat it as so and add the value to the table manually.
		for k,v in pairs(factionTable) do
			--does this entry's first table entry match the faction to check
			if(v[1] == faction) then
				table.remove(factionTable,k)
				player:SetObjVar("Systems_Factions",factionTable)
				return
			end
		end		
	end,
	-----------------------------------------------------------------------------------
	-- 						Systems_Factions.HasFaction
	-----------------------------------------------------------------------------------
	-- player  		- player to check (object)
	-- faction 		- faction to check (string value)
	HasFaction = function(player,faction)
		--grab a local copy of our players System.Factions information
		local factionTable = player:GetObjVar("Systems_Factions") or nil
		--if table is nil then create it
		if(factionTable == nil) then player:SetObjVar("Systems_Factions",{}) end
		--if we have any bad values just return
		if(faction == nil) then return end
		--if the faction table is empty, then this must be the first entry 
		--so treat it as so and add the value to the table manually.
		for k,v in pairs(factionTable) do
			--does this entry's first table entry match the faction to check
			if(v[1] == faction) then
				return true
			end
		end
		return false		
	end,
	-----------------------------------------------------------------------------------
	-- 						Systems_Factions.GetFactionData
	-----------------------------------------------------------------------------------
	-- player  		- player to check (object)
	-- faction 		- faction to check (string value)
	-- return 		- {faction,value} or {"NotFound",-1} if not found in table
	GetFactionRank = function(player,faction)
		--grab a local copy of our players System.Factions information
		local factionTable = player:GetObjVar("Systems_Factions") or nil
		--if table is nil then create it
		if(factionTable == nil) then player:SetObjVar("Systems_Factions",{}) end
		--if we have any bad values just return
		if(faction == nil) then return end
		--if the faction table is empty, then this must be the first entry 
		--so treat it as so and add the value to the table manually.
		for k,v in pairs(factionTable) do
			--does this entry's first table entry match the faction to check
			if(v[1] == faction) then
				return v
			end
		end
		return false	
	end,
}